/*
 * Copyright (c) 2021 Goldman Sachs.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v. 1.0 which accompany this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 */

package org.eclipse.collections.api.factory.map.primitive;

import org.eclipse.collections.api.block.function.primitive.IntFunction;
import org.eclipse.collections.api.block.function.primitive.FloatFunction;
import org.eclipse.collections.api.map.primitive.MutableIntFloatMap;
import org.eclipse.collections.api.map.primitive.IntFloatMap;

/**
 * A factory which creates instances of type {@link MutableIntFloatMap}.
 * This file was automatically generated from template file mutablePrimitivePrimitiveMapFactory.stg.
 *
 * @since 6.0.
 */
public interface MutableIntFloatMapFactory
{
    MutableIntFloatMap empty();

    /**
     * Same as {@link #empty()}.
     */
    MutableIntFloatMap of();

    /**
     * Same as {@link #empty()}.
     */
    MutableIntFloatMap with();

    /**
     * Same as {@link #empty()}. but takes in an initial capacity
     */
    MutableIntFloatMap ofInitialCapacity(int capacity);

    /**
     * Same as {@link #empty()}. but takes in an initial capacity
     */
    MutableIntFloatMap withInitialCapacity(int capacity);

    /**
     * Same as {@link #withAll(IntFloatMap)}.
     */
    MutableIntFloatMap ofAll(IntFloatMap map);

    MutableIntFloatMap withAll(IntFloatMap map);

    /**
     * Creates an {@code MutableIntFloatMap} from an {@code Iterable<T>} by applying {@code keyFunction} and {@code valueFunction}.
     *
     * @since 10.0
     */
    <T> MutableIntFloatMap from(Iterable<T> iterable, IntFunction<? super T> keyFunction, FloatFunction<? super T> valueFunction);
}
